package javax.swing.plaf.synth;

import java.awt.Component;
import javax.swing.border.Border;
import javax.swing.DefaultListCellRenderer;
import java.awt.Color;
import javax.swing.plaf.UIResource;
import javax.swing.JList;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicListUI;

public class SynthListUI extends BasicListUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private boolean useListColors;
    private boolean useUIBorder;
    
    public SynthListUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthListUI();
    }
    
    @Override
    public void update(Graphics g, JComponent c) {
        SynthContext context = this.getContext(c);
        SynthLookAndFeel.update(context, g);
        context.getPainter().paintListBackground(context, g, 0, 0, c.getWidth(), c.getHeight());
        context.dispose();
        this.paint(g, c);
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintListBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.list.addPropertyChangeListener(this);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JList)propertyChangeEvent.getSource());
        }
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.list.removePropertyChangeListener(this);
    }
    
    @Override
    protected void installDefaults() {
        if (this.list.getCellRenderer() == null || this.list.getCellRenderer() instanceof UIResource) {
            this.list.setCellRenderer(new SynthListUI.SynthListCellRenderer());
        }
        this.updateStyle(this.list);
    }
    
    private void updateStyle(JComponent component) {
        SynthContext context = this.getContext(this.list, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style) {
            context.setComponentState(512);
            Color selectionBackground = this.list.getSelectionBackground();
            if (selectionBackground == null || selectionBackground instanceof UIResource) {
                this.list.setSelectionBackground(this.style.getColor(context, ColorType.TEXT_BACKGROUND));
            }
            Color selectionForeground = this.list.getSelectionForeground();
            if (selectionForeground == null || selectionForeground instanceof UIResource) {
                this.list.setSelectionForeground(this.style.getColor(context, ColorType.TEXT_FOREGROUND));
            }
            this.useListColors = this.style.getBoolean(context, "List.rendererUseListColors", true);
            this.useUIBorder = this.style.getBoolean(context, "List.rendererUseUIBorder", true);
            int int1 = this.style.getInt(context, "List.cellHeight", -1);
            if (int1 != -1) {
                this.list.setFixedCellHeight(int1);
            }
            if (style != null) {
                this.uninstallKeyboardActions();
                this.installKeyboardActions();
            }
        }
        context.dispose();
    }
    
    @Override
    protected void uninstallDefaults() {
        super.uninstallDefaults();
        SynthContext context = this.getContext(this.list, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        return SynthLookAndFeel.getComponentState(component);
    }
    
    private class SynthListCellRenderer extends DefaultListCellRenderer.UIResource
    {
        @Override
        public String getName() {
            return "List.cellRenderer";
        }
        
        @Override
        public void setBorder(Border border) {
            if (SynthListUI.this.useUIBorder || border instanceof SynthBorder) {
                super.setBorder(border);
            }
        }
        
        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            if (!SynthListUI.this.useListColors && (isSelected || cellHasFocus)) {
                SynthLookAndFeel.setSelectedUI((SynthLabelUI)SynthLookAndFeel.getUIOfType(this.getUI(), SynthLabelUI.class), isSelected, cellHasFocus, list.isEnabled(), false);
            }
            else {
                SynthLookAndFeel.resetSelectedUI();
            }
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            return this;
        }
        
        @Override
        public void paint(Graphics g) {
            super.paint(g);
            SynthLookAndFeel.resetSelectedUI();
        }
    }
}
